﻿using UnityEngine;
using UnityStandardAssets.ImageEffects;
using System.Collections;
using System.Collections.Generic;

using UnityEngine.SceneManagement;


public class EnviormentHandler : MonoBehaviour
{
	public EviormentalStyle currentEviormentalStyle;


	[Header ("MainSettings")]
	public bool debugMode = false;
	public bool inChangeMode = false;
	[Range (0, 1)]
	public float iOSadjuster;

	[Header ("Depentensies Settings")]
	public Camera mainCamera;


	//public Material seaMat;
	public Material skyMat;

	//public Light SunLight;

	public Color color1;
	public Color color2;
	public Color color3;
	public Color color4;
	public Color color5;


	public List<AtmosphereColorEnforcer> sceneColorEnforcers = new List<AtmosphereColorEnforcer> ();

	public static EnviormentHandler instance;




	public void Awake ()
	{
		instance = this;

	}


	public void AddColorEnforcer (AtmosphereColorEnforcer colorEnforcer)
	{
		sceneColorEnforcers.Add (colorEnforcer);
	}



	void Start ()
	{
		SetSettings (1);
	}

	void FixedUpdate ()
	{

		if (Input.GetKeyDown (KeyCode.Return))
			SceneManager.LoadScene (SceneManager.GetActiveScene ().name);

		if (debugMode)
			SetSettings (currentEviormentalStyle.transitionSmoothnes);
	}

	[ContextMenu ("SetSettings")]
	public void SetSettings ()
	{
		SetSettings (1);
	}


	public void SetSettings (float smoothVal)
	{
		if (debugMode)
			smoothVal *= 100;

		RenderSettings.fogColor = Color.Lerp (RenderSettings.fogColor, currentEviormentalStyle.fogColor, smoothVal);
		RenderSettings.fogStartDistance = Mathf.Lerp (RenderSettings.fogStartDistance, currentEviormentalStyle.fogStartDistance, smoothVal);
		RenderSettings.fogEndDistance = Mathf.Lerp (RenderSettings.fogEndDistance, currentEviormentalStyle.fogEndDistance, smoothVal);

		//SunLight.color = Color.Lerp (SunLight.color, currentEviormentalStyle.sunColor, smoothVal);
		//SunLight.intensity = Mathf.Lerp (SunLight.intensity, currentEviormentalStyle.sunIntensity, smoothVal);

		myTransform.rotation = Quaternion.Slerp (myTransform.rotation, Quaternion.Euler (currentEviormentalStyle.sunRotation), smoothVal);

		//sunShafts.sunThreshold = Color.Lerp (sunShafts.sunThreshold, currentEviormentalStyle.sunShaftThreshold, smoothVal);
		//sunShafts.sunColor = Color.Lerp (sunShafts.sunColor, currentEviormentalStyle.sunShaftColor, smoothVal);

		//seaMat.SetColor ("_Color_Bl", Color.Lerp (seaMat.GetColor ("_Color_Bl"), currentEviormentalStyle.seaDeepColor, smoothVal));
		//seaMat.SetColor ("_Color_A", Color.Lerp (seaMat.GetColor ("_Color_A"), currentEviormentalStyle.seaShallowColor, smoothVal));
		//seaMat.SetColor ("_SeaSideColor", Color.Lerp (seaMat.GetColor ("_SeaSideColor"), currentEviormentalStyle.seaSideColor, smoothVal));
		//seaMat.SetColor ("_SeaEdgeFoamColor", Color.Lerp (seaMat.GetColor ("_SeaEdgeFoamColor"), currentEviormentalStyle.seaEdgeColor, smoothVal));
		//seaMat.SetColor ("_TopFoamColor", Color.Lerp (seaMat.GetColor ("_TopFoamColor"), currentEviormentalStyle.seaFoamColor, smoothVal));
		//seaMat.SetFloat ("_EmisionAmount", Mathf.Lerp (seaMat.GetFloat ("_EmisionAmount"), currentEviormentalStyle.seaEmisionAmount, smoothVal));
		//seaMat.SetFloat ("_SeaDepthColor", Mathf.Lerp (seaMat.GetFloat ("_SeaDepthColor"), currentEviormentalStyle.seaDepthBalance, smoothVal));


		skyMat.SetColor ("_SkyColor", Color.Lerp (skyMat.GetColor ("_SkyColor"), currentEviormentalStyle.horizonEndColor, smoothVal));
		skyMat.SetColor ("_HorizonColor", Color.Lerp (skyMat.GetColor ("_HorizonColor"), currentEviormentalStyle.horizonStartColor, smoothVal));

		skyMat.SetColor ("_CloudColor1", Color.Lerp (skyMat.GetColor ("_CloudColor1"), currentEviormentalStyle.cloudColor, smoothVal));
		skyMat.SetFloat ("_CloudAmount1", Mathf.Lerp (skyMat.GetFloat ("_CloudAmount1"), currentEviormentalStyle.cloudAmount, smoothVal));
		skyMat.SetFloat ("_CloudsScrollSpeed1", Mathf.Lerp (skyMat.GetFloat ("_CloudsScrollSpeed1"), currentEviormentalStyle.cloudSpeed, smoothVal));


		skyMat.SetColor ("_CloudColor2", Color.Lerp (skyMat.GetColor ("_CloudColor2"), currentEviormentalStyle.cloudColor2, smoothVal));
		skyMat.SetFloat ("_CloudAmount2", Mathf.Lerp (skyMat.GetFloat ("_CloudAmount2"), currentEviormentalStyle.cloudAmount2, smoothVal));
		skyMat.SetFloat ("_CloudsScrollSpeed2", Mathf.Lerp (skyMat.GetFloat ("_CloudsScrollSpeed2"), currentEviormentalStyle.cloudSpeed2, smoothVal));


		//seaMat.SetColor ("_SpecularColor", Color.Lerp (seaMat.GetColor ("_SpecularColor"), currentEviormentalStyle.glossColor, smoothVal));
		//seaMat.SetFloat ("_Gloss", Mathf.Lerp (seaMat.GetFloat ("_Gloss"), currentEviormentalStyle.glossAmount, smoothVal));


		color1 = Color.Lerp (color1, currentEviormentalStyle.color1, smoothVal);
		color2 = Color.Lerp (color2, currentEviormentalStyle.color2, smoothVal);
		color3 = Color.Lerp (color3, currentEviormentalStyle.color3, smoothVal);
		color4 = Color.Lerp (color4, currentEviormentalStyle.color4, smoothVal);
		color5 = Color.Lerp (color5, currentEviormentalStyle.color5, smoothVal);

		for (int i = 0; i < sceneColorEnforcers.Count; i++)
			sceneColorEnforcers [i].UpdateColorSettings ();
	}




	public void ChangeEnviormentalStyle (EviormentalStyle newEviormentalStyle, float duration, EaseType easer)
	{
		currentEviormentalStyle = newEviormentalStyle;

		StartCoroutine (TransitionSettings (duration, easer));
	}


	IEnumerator TransitionSettings (float duration, EaseType easer)
	{

		inChangeMode = true;

		var startFogColor = RenderSettings.fogColor;
		var startFogStartDistance = RenderSettings.fogStartDistance;
		var startFogEndDistance = RenderSettings.fogEndDistance;

		//var startSunLightColor = SunLight.color;
		//var startSunLightIntensity = SunLight.intensity;

		Quaternion startSunRotation = myTransform.rotation;


		//var startSunShaftsSunThreshold = sunShafts.sunThreshold;
		//var startSunShaftsSunColor = sunShafts.sunColor;

		//var startSeaColorDepth = seaMat.GetColor ("_Color_Bl");
		//var startSeaColorEdge = seaMat.GetColor ("_Color_A");
		//var startSeaSideColor = seaMat.GetColor ("_SeaSideColor");
		//var startSeaEdgeFoamColor = seaMat.GetColor ("_SeaEdgeFoamColor");
		//var startSeaTopFoamColor = seaMat.GetColor ("_TopFoamColor");

		//float startEmisionAmount = seaMat.GetFloat ("_EmisionAmount");
		//float startSeaDepthColor = seaMat.GetFloat ("_SeaDepthColor");

		//Color startGlossColor = seaMat.GetColor ("_SpecularColor");
		//float startGlossAmount = seaMat.GetFloat ("_Gloss");



		var startSkyColor = skyMat.GetColor ("_SkyColor");
		var startHorizonColor = skyMat.GetColor ("_HorizonColor");

		var startCloudColor = skyMat.GetColor ("_CloudColor1");
		float startCloudAmount = skyMat.GetFloat ("_CloudAmount1");
		float startCloudsScrollSpeed = skyMat.GetFloat ("_CloudsScrollSpeed1");


		var startCloudColor2 = skyMat.GetColor ("_CloudColor2");
		float startCloudAmount2 = skyMat.GetFloat ("_CloudAmount2");
		float startCloudsScrollSpeed2 = skyMat.GetFloat ("_CloudsScrollSpeed2");




		Color startColor1 = color1;
		Color startColor2 = color2;
		Color startColor3 = color3;
		Color startColor4 = color4;
		Color startColor5 = color5;


		float elapsed = 0;
		float start = 0;
		float range = 1;
		float time = 0;

		Easer ease = Ease.FromType (easer);

		while (elapsed < duration) {
			elapsed = Mathf.MoveTowards (elapsed, duration, Time.deltaTime);
			time = start + range * ease (elapsed / duration);

			RenderSettings.fogColor = Color.Lerp (startFogColor, currentEviormentalStyle.fogColor, time);
			RenderSettings.fogStartDistance = Mathf.Lerp (startFogStartDistance, currentEviormentalStyle.fogStartDistance, time);
			RenderSettings.fogEndDistance = Mathf.Lerp (startFogEndDistance, currentEviormentalStyle.fogEndDistance, time);


			//SunLight.color = Color.Lerp (startSunLightColor, currentEviormentalStyle.sunColor, time);
			//	SunLight.intensity = Mathf.Lerp (startSunLightIntensity, currentEviormentalStyle.sunIntensity, time);
			//myTransform.rotation = Quaternion.Slerp (startSunRotation, Quaternion.Euler (currentEviormentalStyle.sunRotation), time);



			//sunShafts.sunThreshold = Color.Lerp (startSunShaftsSunThreshold, currentEviormentalStyle.sunShaftThreshold, time);
			//sunShafts.sunColor = Color.Lerp (startSunShaftsSunColor, currentEviormentalStyle.sunShaftColor, time);


			//seaMat.SetColor ("_Color_Bl", Color.Lerp (startSeaColorDepth, currentEviormentalStyle.seaDeepColor, time));
			//seaMat.SetColor ("_Color_A", Color.Lerp (startSeaColorEdge, currentEviormentalStyle.seaShallowColor, time));
			/*
			seaMat.SetColor ("_SeaSideColor", Color.Lerp (startSeaSideColor, currentEviormentalStyle.seaSideColor, time));
			seaMat.SetColor ("_SeaEdgeFoamColor", Color.Lerp (startSeaEdgeFoamColor, currentEviormentalStyle.seaEdgeColor, time));
			seaMat.SetColor ("_TopFoamColor", Color.Lerp (startSeaTopFoamColor, currentEviormentalStyle.seaFoamColor, time));

			seaMat.SetFloat ("_EmisionAmount", Mathf.Lerp (startEmisionAmount, currentEviormentalStyle.seaEmisionAmount, time));
			seaMat.SetFloat ("_SeaDepthColor", Mathf.Lerp (startSeaDepthColor, currentEviormentalStyle.seaDepthBalance, time));

			seaMat.SetColor ("_SpecularColor", Color.Lerp (startGlossColor, currentEviormentalStyle.glossColor, time));
			seaMat.SetFloat ("_Gloss", Mathf.Lerp (startGlossAmount, currentEviormentalStyle.glossAmount, time));
*/


			skyMat.SetColor ("_SkyColor", Color.Lerp (startSkyColor, currentEviormentalStyle.horizonEndColor, time));
			skyMat.SetColor ("_HorizonColor", Color.Lerp (startHorizonColor, currentEviormentalStyle.horizonStartColor, time));

			skyMat.SetColor ("_CloudColor1", Color.Lerp (startCloudColor, currentEviormentalStyle.cloudColor, time));
			skyMat.SetFloat ("_CloudAmount1", Mathf.Lerp (startCloudAmount, currentEviormentalStyle.cloudAmount, time));
			skyMat.SetFloat ("_CloudsScrollSpeed1", Mathf.Lerp (startCloudsScrollSpeed, currentEviormentalStyle.cloudSpeed, time));

			skyMat.SetColor ("_CloudColor2", Color.Lerp (startCloudColor2, currentEviormentalStyle.cloudColor2, time));
			skyMat.SetFloat ("_CloudAmount2", Mathf.Lerp (startCloudAmount2, currentEviormentalStyle.cloudAmount2, time));
			skyMat.SetFloat ("_CloudsScrollSpeed2", Mathf.Lerp (startCloudsScrollSpeed2, currentEviormentalStyle.cloudSpeed2, time));


			color1 = Color.Lerp (startColor1, currentEviormentalStyle.color1, time);
			color2 = Color.Lerp (startColor2, currentEviormentalStyle.color2, time);
			color3 = Color.Lerp (startColor3, currentEviormentalStyle.color3, time);
			color4 = Color.Lerp (startColor4, currentEviormentalStyle.color4, time);
			color5 = Color.Lerp (startColor5, currentEviormentalStyle.color5, time);

			for (int i = 0; i < sceneColorEnforcers.Count; i++) {
				sceneColorEnforcers [i].UpdateColorSettings ();
			}

			yield return 0;
		}

		inChangeMode = false;
	}



	public void SetAtmosphere (EviormentalStyle  from, EviormentalStyle to, float time)
	{

		RenderSettings.fogColor = Color.Lerp (from.fogColor, to.fogColor, time);
		RenderSettings.fogStartDistance = Mathf.Lerp (from.fogStartDistance, to.fogStartDistance, time);
		RenderSettings.fogEndDistance = Mathf.Lerp (from.fogEndDistance, to.fogEndDistance, time);


		//SunLight.color = Color.Lerp (from.sunColor, to.sunColor, time);
		//SunLight.intensity = Mathf.Lerp (from.sunIntensity, to.sunIntensity, time);
		//myTransform.rotation = Quaternion.Slerp (Quaternion.Euler (from.sunRotation), Quaternion.Euler (to.sunRotation), time);



		//seaMat.SetColor ("_Color_Bl", Color.Lerp (from.seaDeepColor, to.seaDeepColor, time));
		//seaMat.SetColor ("_Color_A", Color.Lerp (from.seaShallowColor, to.seaShallowColor, time));
		/*
		seaMat.SetColor ("_SeaSideColor", Color.Lerp (from.seaSideColor, to.seaSideColor, time));
		seaMat.SetColor ("_SeaEdgeFoamColor", Color.Lerp (from.seaEdgeColor, to.seaEdgeColor, time));
		seaMat.SetColor ("_TopFoamColor", Color.Lerp (from.seaFoamColor, to.seaFoamColor, time));

		seaMat.SetFloat ("_EmisionAmount", Mathf.Lerp (from.seaEmisionAmount, to.seaEmisionAmount, time));
		seaMat.SetFloat ("_SeaDepthColor", Mathf.Lerp (from.seaDepthBalance, to.seaDepthBalance, time));

		seaMat.SetColor ("_SpecularColor", Color.Lerp (from.glossColor, to.glossColor, time));
		seaMat.SetFloat ("_Gloss", Mathf.Lerp (from.glossAmount, to.glossAmount, time));
*/



		skyMat.SetColor ("_SkyColor", Color.Lerp (from.horizonEndColor, to.horizonEndColor, time));
		skyMat.SetColor ("_HorizonColor", Color.Lerp (from.horizonStartColor, to.horizonStartColor, time));

		skyMat.SetColor ("_CloudColor1", Color.Lerp (from.cloudColor, to.cloudColor, time));
		skyMat.SetFloat ("_CloudAmount1", Mathf.Lerp (from.cloudAmount, to.cloudAmount, time));
		skyMat.SetFloat ("_CloudsScrollSpeed1", Mathf.Lerp (from.cloudSpeed, to.cloudSpeed, time));

		skyMat.SetColor ("_CloudColor2", Color.Lerp (from.cloudColor2, to.cloudColor2, time));
		skyMat.SetFloat ("_CloudAmount2", Mathf.Lerp (from.cloudAmount2, to.cloudAmount2, time));
		skyMat.SetFloat ("_CloudsScrollSpeed2", Mathf.Lerp (from.cloudSpeed2, to.cloudSpeed2, time));





		color1 = Color.Lerp (from.color1, to.color1, time);
		color2 = Color.Lerp (from.color2, to.color2, time);
		color3 = Color.Lerp (from.color3, to.color3, time);
		color4 = Color.Lerp (from.color4, to.color4, time);
		color5 = Color.Lerp (from.color5, to.color5, time);

		for (int i = 0; i < sceneColorEnforcers.Count; i++) {
			sceneColorEnforcers [i].UpdateColorSettings ();
		}
	}


	private Transform _myTransform;

	private Transform myTransform {
		get {
			if (_myTransform == null)
				_myTransform = GetComponent<Transform> ();
			return _myTransform;
		}

		set {
			_myTransform = value;
		}
	}

	/*
	private SunShafts _sunShafts;

	private SunShafts sunShafts {
		get {
			if (_sunShafts == null)
				_sunShafts = mainCamera.GetComponent<SunShafts> ();
			return _sunShafts;
		}   
		set {
			_sunShafts = value;
		}
	}*/

}