using UnityEngine;
using AudioSystem;
using System.Collections;

namespace AudioSystem
{
		/// <summary>
		/// The Audio helper class create the nessecery object for the audio playr.
		/// Could be extedend in the future to hanlde object Pooling
		/// </summary>
		public class AudioHelper : MonoBehaviour
		{

				private static GameObject _soundGO;

				public static GameObject soundGO {
						get {
								if (_soundGO == null) {
										_soundGO = new GameObject ();
										_soundGO.name = "SoundContainer";
								}
								return _soundGO;
						}   
						set {
								_soundGO = value;
						}
				}


				// over loads
				public static void CreatePlayAudioObject (AudioClip aClip)
				{
						CreatePlayAudioObject (aClip, 1.0f);
				}

				public static void CreatePlayAudioObject (AudioClip aClip, float vol)
				{
						CreatePlayAudioObject (aClip, vol, "AudioPlayObject");
				}
				// creates a dynamic Audio object to position and play in the world
				public static void CreatePlayAudioObject (AudioClip aClip, float vol, string objName, float pitch = 1)
				{
						// instance a new gameobject
						GameObject apObject = new GameObject (objName);
						// position the object in the world
						apObject.transform.position = Vector3.zero;
						apObject.transform.parent = soundGO.transform;
						// add an AudioSource component
						apObject.AddComponent<AudioSource> ();
						// return this script for use
						AudioSource apAudio = apObject.GetComponent<AudioSource> ();
						// initialize some AudioSource fields
						apAudio.playOnAwake = false;
						apAudio.rolloffMode = AudioRolloffMode.Linear;
						apAudio.pitch = pitch;
						apAudio.loop = false;		
						apAudio.clip = aClip;
						apAudio.volume = vol;
		
						// play the clip
						apAudio.Play ();
						// destroy this object after clip length
						Destroy (apObject, aClip.length);
				}
	
				// fade our AudioSource object based on speed (> 0 fades volume up, < 0 fades volume out, == 0 assumes the sound is playing and just destroys it)
				public static IEnumerator FadeAudioObject (GameObject aObject, float fadeSpeed)
				{
						Animation apAnim = aObject.GetComponent<Animation> ();
						AudioSource aSource = aObject.GetComponent<AudioSource> ();

						/*
				// we are not a fade audio object
				if (apAnim == null) { 
						// we simply destroy the object and return
						if (fadeSpeed <= 0) {
								Destroy (aObject);
						}

						// we are a psitive playing sound, so just play it
						if (fadeSpeed > 0 && aSource != null) {
								aSource.Play ();
						}

						return true;
				}*/

						// animation clip is default to fade out (1 to 0), these will look reveresed but they are correct
						if (fadeSpeed < 0) { 
								apAnim [apAnim.clip.name].time = apAnim [apAnim.clip.name].length; 
						} else { 
								apAnim [apAnim.clip.name].time = 0; 
						}
		
						// set our speed
						apAnim [apAnim.clip.name].speed = fadeSpeed; 
		
						// play the audio
						if (aSource.isPlaying == false) {
								aSource.Play ();
						}		
		
						// play the fade
						apAnim.Play ();
	
						// yield the length of the clip
						if (fadeSpeed < 0) {
								while (apAnim.isPlaying) {
										yield return new WaitForEndOfFrame ();
								}
			
								Destroy (aObject);
						}	
				}
	
				// over loads
				public static GameObject CreateGetFadeAudioObject (AudioClip aClip)
				{
						return CreateGetFadeAudioObject (aClip, true);
				}

				public static GameObject CreateGetFadeAudioObject (AudioClip aClip, bool dLoop)
				{
						return CreateGetFadeAudioObject (aClip, dLoop, null);
				}

				public static GameObject CreateGetFadeAudioObject (AudioClip aClip, bool dLoop, AnimationClip fadeClip)
				{
						return CreateGetFadeAudioObject (aClip, dLoop, fadeClip, "ReturnedAudioPlayObject");
				}
	
				// creates/returns a dynamic Audio object to position and plays in the world with loop
				public static GameObject CreateGetFadeAudioObject (AudioClip aClip, bool dLoop, AnimationClip fadeClip, string objName)
				{
						// instance a new gameobject
						GameObject apObject = new GameObject (objName);
						// position the object in the world
						apObject.transform.position = Vector3.zero;
						apObject.transform.parent = soundGO.transform;
						// add our DontDestroyOnLoad
						//DontDestroyOnLoad (apObject);		
						// add an AudioSource component
						AudioSource apAudio = apObject.AddComponent<AudioSource> ();
						// initialize some AudioSource fields
						apAudio.loop = dLoop;		
						apAudio.clip = aClip;
			
						// return our AudioObject
						return apObject;
				}
		}
}
