﻿using UnityEngine;
using AudioSystem;
using UnityEditor;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using System;

[CustomEditor (typeof(AudioManager))]
/// <summary>
/// The Audio manager editor Tool extends the AudioManager and provide a more user friendly interface for it.
/// is allow the editing of the soudns but also provides a drag and drop section can handler audio file and create their slots.
/// </summary>
public class AudioManagerEditor : Editor
{
	
		Color defaultColor;


		private AudioManager _myAudioManager;

		private AudioManager myAudioManager {
				get {
						if (_myAudioManager == null)
								_myAudioManager = (AudioManager)target;
						return _myAudioManager;
				}   
				set {
						_myAudioManager = value;
				}
		}



		void OnEnable ()
		{
				
		}

		public override void OnInspectorGUI ()
		{

				defaultColor = GUI.color;

				EditorGUILayout.LabelField ("Settings");

				myAudioManager.isMuted = EditorGUILayout.Toggle ("Sound Muted", myAudioManager.isMuted);
			

				GUI.color = new Color (99f / 255f, 197f / 255f, 255f / 255f);
				EditorGUILayout.BeginVertical ("Box");
				GUI.color = defaultColor;
		
				EditorGUILayout.LabelField ("Sfx Settings");
		
				for (int i = 0; i < myAudioManager.audioList.Count; i++)
						DrawAudioClassElement (myAudioManager.audioList [i]);
		
				EditorGUILayout.EndVertical ();
		
				DropAreaGUISound ();


				GUI.color = new Color (86f / 255f, 176f / 255f, 0f);
				EditorGUILayout.BeginVertical ("Box");
				GUI.color = defaultColor;

				EditorGUILayout.LabelField ("Music Settings");
				
				for (int i = 0; i < myAudioManager.musicList.Count; i++) {
						DrawAudioClassElement (myAudioManager.musicList [i]);
				}
				
				EditorGUILayout.EndVertical ();

				DropAreaGUIMusic ();



		}




		public void DrawAudioClassElement (AudioClass audioClass)
		{
				string editBtnTitle = "Edit";
				if (audioClass.isOpen) {
						GUI.color = new Color (130f / 255f, 195f / 255f, 26f / 255f);
						editBtnTitle = "Close";
				}
			

				EditorGUILayout.BeginVertical ("box");

				GUI.color = defaultColor;

				EditorGUILayout.BeginHorizontal ();


				if (GUILayout.Button (editBtnTitle, GUILayout.Width (50))) {
						audioClass.isOpen = !audioClass.isOpen;
				}
			
				
				if (GUILayout.Button ("S", GUILayout.Width (30)))
						PlayClip (audioClass.audioClip);

				EditorGUILayout.LabelField (audioClass.id);

				if (audioClass.isOpen) {
						GUI.color = Color.red;
						if (GUILayout.Button ("X", GUILayout.Width (30)))
								myAudioManager.audioList.Remove (audioClass);

						GUI.color = defaultColor;
						
				}

				EditorGUILayout.EndHorizontal ();

				if (audioClass.isOpen) {
						EditorGUILayout.BeginHorizontal ();
						audioClass.id = EditorGUILayout.TextField (audioClass.id);
						audioClass.audioClip = (AudioClip)EditorGUILayout.ObjectField (audioClass.audioClip, typeof(AudioClip), true /*, GUILayout.Width (160)*/); 
						EditorGUILayout.EndHorizontal ();

						EditorGUILayout.BeginHorizontal ();
						audioClass.volume = EditorGUILayout.Slider (audioClass.volume, 0, 1);
						EditorGUILayout.EndHorizontal ();
						
				} 

				EditorGUILayout.EndVertical ();
				GUI.color = defaultColor;
		}






		public void DropAreaGUISound ()
		{
				Event evt = Event.current;
				Rect drop_area = GUILayoutUtility.GetRect (0.0f, 50.0f, GUILayout.ExpandWidth (true));
				GUI.Box (drop_area, "Add Sound");
		
				switch (evt.type) {
				case EventType.DragUpdated:
				case EventType.DragPerform:
						if (!drop_area.Contains (evt.mousePosition))
								return;
			
						DragAndDrop.visualMode = DragAndDropVisualMode.Copy;
			
						if (evt.type == EventType.DragPerform) {
								DragAndDrop.AcceptDrag ();
				
								foreach (UnityEngine.Object dragged_object in DragAndDrop.objectReferences) {
										Debug.Log (string.Format ("dragged_object : {0}", dragged_object.name));
										if (dragged_object.GetType () == typeof(AudioClip)) {
												AudioClip audioClip = (AudioClip)dragged_object;
												AudioClass newaudioClass = new AudioClass (audioClip);
												newaudioClass.isOpen = true;
												myAudioManager.audioList.Add (newaudioClass);
										}
								}
						}
						break;
				}
		}




		public void DropAreaGUIMusic ()
		{
				Event evt = Event.current;
				Rect drop_area = GUILayoutUtility.GetRect (0.0f, 50.0f, GUILayout.ExpandWidth (true));
				GUI.Box (drop_area, "Add Music");
		
				switch (evt.type) {
				case EventType.DragUpdated:
				case EventType.DragPerform:
						if (!drop_area.Contains (evt.mousePosition))
								return;
			
						DragAndDrop.visualMode = DragAndDropVisualMode.Copy;
			
						if (evt.type == EventType.DragPerform) {
								DragAndDrop.AcceptDrag ();
				
								foreach (UnityEngine.Object dragged_object in DragAndDrop.objectReferences) {
										Debug.Log (string.Format ("dragged_object : {0}", dragged_object.name));
										if (dragged_object.GetType () == typeof(AudioClip)) {
												AudioClip audioClip = (AudioClip)dragged_object;
												AudioClass newaudioClass = new AudioClass (audioClip);
												newaudioClass.isOpen = true;
												myAudioManager.musicList.Add (newaudioClass);
										}
								}
						}
						break;
				}
		}

	
	





		public static void PlayClip (AudioClip clip)
		{
				Assembly unityEditorAssembly = typeof(AudioImporter).Assembly;
				Type audioUtilClass = unityEditorAssembly.GetType ("UnityEditor.AudioUtil");
				MethodInfo method = audioUtilClass.GetMethod ("PlayClip", BindingFlags.Static | BindingFlags.Public, null,
						                    new System.Type[] {
								typeof(AudioClip)
						},
						                    null
				                    );
				method.Invoke (
						null,
						new object[] {
								clip
						}
				);
		}
}







