﻿using UnityEngine;
using System.Collections;

public class CameraFollow : MonoBehaviour
{

	[Header ("General Settings")]
	public Transform target;

	public CameraSettingsClass cameraSettings;

	public bool dontSink = false;

	public float shakeDropOff;

	public bool extraFollowOverwrite = true;

	[HideInInspector]
	public Vector3 followpos;

	[HideInInspector]
	public float shakeGuide = 0;



	public static CameraFollow instance;

	void Awake ()
	{
		instance = this;
	}

	void Start ()
	{
		if (target)
			followpos = getTargetPosition ();
	}


	public void FixEvents ()
	{
		Debug.Log (string.Format ("CameraFollow FixEvents()"));
		cameraSettings.followTarget = true;
	}

	public void RemoveEvents ()
	{
		Debug.Log (string.Format ("CameraFollow RemoveEvents()"));
		cameraSettings.followTarget = false;
		target = null;
	}

	void Update ()
	{
		
		if (!cameraSettings.followTarget || !extraFollowOverwrite) {
			followpos = myTransform.position;
			return;
		}

		if (target)
			followpos = getTargetPosition ();

		if (shakeGuide != 0)
			followpos += ShakeOff ();
		

		myTransform.position = Vector3.Lerp (myTransform.position, followpos, cameraSettings.smoothFollow * Time.deltaTime);

		if (dontSink) {
			Vector3 resultPos = myTransform.position;
			float minYpos = OceanHandler.instance.GetHeight (resultPos, false, false);
			if (resultPos.y < minYpos)
				resultPos.y = minYpos;

			myTransform.position = resultPos;
		}



		if (!cameraSettings.rotationLock) {
			Vector3 lookRot = target.rotation.eulerAngles;
			lookRot.y = 0;
			myTransform.rotation = Quaternion.Lerp (myTransform.rotation, Quaternion.Euler (lookRot), cameraSettings.smoothFollowRot * Time.deltaTime);
		} else
			myTransform.rotation = Quaternion.Lerp (myTransform.rotation, Quaternion.Euler (cameraSettings.rotationOffset), cameraSettings.smoothFollowRot * Time.deltaTime);

	}


	public Vector3 ShakeOff ()
	{
		float shakeXamount = Random.Range (-shakeGuide, shakeGuide);
		float shakeYamount = Random.Range (0, shakeGuide * 2);
		float shakeZamount = Random.Range (-shakeGuide, shakeGuide);
		Vector3 shakeAmount = new Vector3 (shakeXamount, shakeYamount, shakeZamount);
		shakeGuide = Mathf.Lerp (shakeGuide, 0, shakeDropOff * Time.deltaTime);
		return  shakeAmount;


	}



	Vector3 getTargetPosition ()
	{
		Vector3 resultPos = cameraSettings.offset + target.position;


		return resultPos;
	}



	public Vector3 position {
		get {
			return myTransform.position;
		}
	}


	private Transform _myTransform;

	private Transform myTransform {
		get {
			if (_myTransform == null)
				_myTransform = GetComponent<Transform> ();
			return _myTransform;
		}

		set {
			_myTransform = value;
		}

	}



}
