﻿using UnityEngine;
using AudioSystem;
using System.Collections;

[System.Serializable]
/// <summary>
/// A Room action contain nessecery information for a timeline Step.
/// </summary>
public class RoomAction : ScriptableObject
{

	[SerializeField]
	public RoomTasksV2
		taskV2;

	[SerializeField]
	public GameObject
		gameItem;

	[SerializeField]
	public bool 
		hideBubble = true;

	[SerializeField]
	public float 
		waitTime = 0f;

	[SerializeField]
	public string 
		Text = "";
		
	[SerializeField]
	public bool
		isTaskListed = false;
	
	[SerializeField]
	public AnimationType
		animType;


	[SerializeField]
	public bool
		ShowItem = true;

	[SerializeField]
	public float 
		walkSpeed = 5f;
		
	[SerializeField]
	public int
		layerID = 0;
	[SerializeField]
	public int
		subLayerid = 0;

	[SerializeField]
	public bool
		isActive = true;

	[SerializeField]
	public int 
		taskID = 0;

	[SerializeField]
	public int 
		subtaskID = 0;


	[SerializeField]
	public int 
		score = 0;

	[SerializeField]
	public string
		description = "";

		

	public static RoomAction DublicateRoomAction (RoomAction original)
	{
		RoomAction clone = ScriptableObject.CreateInstance <RoomAction> ();
		clone.taskV2 = original.taskV2;
		clone.gameItem = original.gameItem;
		clone.hideBubble = original.hideBubble;
		clone.waitTime = original.waitTime;
		clone.Text = original.Text;
		clone.isTaskListed = original.isTaskListed;
		clone.animType = original.animType;
		clone.ShowItem = original.ShowItem;
		clone.walkSpeed = original.walkSpeed;
		clone.layerID = original.layerID;
		clone.subLayerid = original.subLayerid;
		clone.isActive = original.isActive;
		clone.taskID = original.taskID;
		clone.subtaskID = original.subtaskID;
		clone.score = original.score;
		clone.description = original.description;

		return clone;
	}

}





[System.Serializable]
public enum RoomTasks
{
	EnableTask,
	MoveCharacter,
	Animate,
	ShowHelpText,
	ShowQuestion,
	ShowInvetory,
	ShowTopBar,
	HideTopBar,
	ActivateObject,
	DeactivateObject,
	Transition,
	PlayAnimation,
	InsertAboveLayer,
	InsertBelowLayer,
	CrawlCharacter,
	SetLayer,
	PlaySoundFX,
	RewindFX,
	Enlisted,
	SetTask,
	SetSubTask,
	CameraFadeIn,
	CameraFadeOut,
	none,
}




[System.Serializable]
[System.Flags]
/// <summary>
/// Defines the type of Scenarion steps
/// </summary>
public enum RoomTasksV2
{
	none = 1 << 0,
	Enlisted = 1 << 1,
	EnableTask = 1 << 2,

	CrawlCharacter = 1 << 5,
	MoveCharacter = 1 << 6,

	ActivateObject = 1 << 10,
	DeactivateObject = 1 << 11,

	Transition = 1 << 15,
	RewindFX = 1 << 16,
	PlaySoundFX = 1 << 17,

	ShowHelpText = 1 << 20,
	ShowQuestion = 1 << 21,
	ShowInvetory = 1 << 22,

	ShowTopBar = 1 << 25,
	HideTopBar = 1 << 26,

	SetTask = 1 << 30,
	SetSubTask = 1 << 31,

	CameraFadeIn = 1 << 35,
	CameraFadeOut = 1 << 36,

	GroupStart = 1 << 40,
	GroupEnd = 1 << 41
} 