﻿using UnityEngine;
using AudioSystem;
using System.Collections;
using System.Collections.Generic;

/// <summary>
/// The Mutliple correct position is a tool for making puzzles tha have drag and drop objects but their correct position can be more then one.
/// At the moment this tool has been used for the fire Triangle and has a small referece to it but should be ok to be reused.
/// </summary>
public class MutlipleCorrectPos : TaskHandler
{
	/// <summary>
	/// List of all location fot correct positions.
	/// </summary>
	public List<Transform> CorrectPositions;
	/// <summary>
	/// The maximum allowed wining distance.
	/// </summary>
	public float maxWinDistance;

	/// <summary>
	/// The feedback display time.
	/// </summary>
	public float feedbackTime = 3f;


	void Awake ()
	{
		OnHout ();
	}

	/// <summary>
	/// Enables the task and its objectives
	/// </summary>
	public override void EnableTask ()
	{
		foreach (DragableItem dragItem in GetComponentsInChildren<DragableItem> ())
			dragItem.gameObject.SendMessage (HRUtilities.EnableObjective);
	}

	/// <summary>
	/// Raise by child elements to show hover points
	/// </summary 
	public void OnHover ()
	{
		for (int i = 0; i < CorrectPositions.Count; i++)
			CorrectPositions [i].gameObject.SetActive (true);
	}

	/// <summary>
	/// Raise by child elements to show hide points
	/// </summary 
	public void OnHout ()
	{
		for (int i = 0; i < CorrectPositions.Count; i++)
			CorrectPositions [i].gameObject.SetActive (false);
	}

	/// <summary>
	/// When an Objective is completed.
	/// </summary>
	/// <param name="posItem">Positioned GameObject</param>
	public void ObjectiveCompleted (GameObject posItem)
	{
		//Inarate throught the correct position list
		for (int i = 0; i < CorrectPositions.Count; i++) {
			// Calculate the distance
			float dist = Vector3.Distance (CorrectPositions [i].position, posItem.transform.position);
			// if with allow range
			if (dist < maxWinDistance) {
				// Get the DragableItem from the object
				DragableItem dragItem = posItem.GetComponent <DragableItem> ();
				// if its set to be from the correct list
				if (dragItem.isInCorrectPosition) {
					// we remove its its evets 
					dragItem.SendMessage (HRUtilities.RemoveEvents);
					// send a congrats msg to the top bar
					TopBarController.ShowObjective (dragItem.goodfeedBackText, true, feedbackTime);
					// animate the object to the correct location
					StartCoroutine (AnimateCorrect (dragItem, CorrectPositions [i].position));
					// remove it from the list
					CorrectPositions.Remove (CorrectPositions [i]);
				} else {
					// if not correct we send it back to its original location
					StartCoroutine (AnimateWrong (dragItem));
					// and diplay the wrong message to the top bar
					TopBarController.ShowObjective (dragItem.badfeedBackText, false, feedbackTime);
				}	

				break;
			} else {
				Debug.Log (string.Format ("Nothing!"));
			}	
		}
	}

	/// <summary>
	/// Animates An object to its correct position
	/// </summary>
	/// <param name="dragItem">Drag item</param>
	/// <param name="finalPos">Final position</param>
	IEnumerator AnimateCorrect (DragableItem dragItem, Vector3 finalPos)
	{
		StartCoroutine (dragItem.transform.ScaleTo (Vector3.one * 1.5f, HRUtilities.at, EaseType.CubeOut));
		yield return StartCoroutine (dragItem.SetToFinalPosition (finalPos, Quaternion.identity));
		yield return StartCoroutine (dragItem.transform.ScaleTo (Vector3.one, HRUtilities.at, EaseType.CubeIn));
		CheckForVictory ();
	}

	/// <summary>
	/// Animates An object to its original position
	/// </summary>
	/// <param name="dragItem">Drag item.</param>
	IEnumerator AnimateWrong (DragableItem dragItem)
	{
		yield return StartCoroutine (dragItem.transform.transform.Shake (0.21f, HRUtilities.at * 2));
		yield return StartCoroutine (Auto.Wait (HRUtilities.at));
		dragItem.MoveToStartPosition ();
		
	}


	/// <summary>
	/// Checks the list if player have found the correct position of all the objects.
	/// </summary>
	private void CheckForVictory ()
	{
		if (CorrectPositions.Count == 0) {
			foreach (DragableItem dragItem in GetComponentsInChildren <DragableItem>())
				dragItem.SendMessage (HRUtilities.RemoveEvents);
						
			SendTaskCompleted ();
		}
	}
		

		
}
