﻿using UnityEngine;
using UnityEngine.EventSystems;
using AudioSystem;
using System.Collections;


/// <summary>
/// The Switch Class is a utility tool for creating puzzles or Tasks with simple one Click interactions
/// It can be use as Task tool or as an objective of a task.
/// with apropreate naming convention it will also handle hovers.
/// </summary>
public class Switch : TaskHandler
{

	private GameObject myGlow;
	//private GameObject myNormal;


	[Header ("Events Settings")]
	/// <summary>
	/// If isObjective set to true then the Switch is a part of a task and will send a "ObjectiveCompleted" when clicked.
	/// </summary>
	public bool isObjective = false;
	/// <summary>
	/// If EnableOnLoad set to true the class will activate its self imidiatly
	/// </summary>
	public bool EnableOnLoad = false;
	/// <summary>
	/// If onClickComplete is set to false the whole tool will only use the hover features.
	/// </summary>
	public bool onClickComplete;
	/// <summary>
	/// The isCorrect Bool is used mainly for the topbar Feedback other for third script to check the condition of this choice
	/// </summary>
	public bool isCorrect = false;

	[Header ("Hover Settings")]
	/// <summary>
	/// With GlowIsSeperated bool you can define if the hover state is an image replacement of a simple object activation
	/// </summary>
	public bool GlowIsSeperated;
	/// <summary>
	/// yeah...
	/// </summary>
	public bool glowIsVisible = false;
	/// <summary>
	/// Topbar FeedBack OnClick Text 
	/// </summary>
	public string FeedBackText;
	/// <summary>
	/// If colliderRemoveFix is set to true then the class will resize the collider and set to zero when tool is inactive.
	/// </summary>
	public bool colliderRemoveFix = false;

	private Vector3 startColliderSize;

	public bool isActive;

	void Start ()
	{
		startColliderSize = myBoxCollider.size;

		///Find And Setup the hove Object
		Transform glowTR = transform.FindChild ("glow");
		if (glowTR) {
			myGlow = glowTR.gameObject;
			if (!glowIsVisible)
				myGlow.SetActive (false);
		}
		// Find normal state Object
		/*
		if (!GlowIsSeperated)
			myNormal = transform.FindChild ("normal").gameObject;
			*/

		if (FeedBackText != "")
			FeedBackText = TextHandler.Instance.GetText (FeedBackText);

		//Enable the Switch Object at start if selected
		if (EnableOnLoad) {
			if (isObjective)
				EnableObjective ();
			else
				EnableTask ();
		}
	}

	/// <summary>
	/// Enables the task.
	/// </summary>
	public override void EnableTask ()
	{
		if (isObjective)
			return;
		FixEvents ();
	}

	/// <summary>
	/// Enables the objective.
	/// </summary>
	public void EnableObjective ()
	{
		if (!isObjective)
			return;
		FixEvents ();
	}

	/// <summary>
	/// Fixs the events.
	/// </summary>
	public void FixEvents ()
	{
		// first we remove the event to make sure their are no dublicates
		RemoveEvents ();

		// handle the collider
		if (colliderRemoveFix)
			myBoxCollider.size = startColliderSize;

		// Add the onClick event
		//uitem.OnClick += Click;

		isActive = true;
	}



	//public void OnPointerClick (PointerEventData eventData)
	public override void OnPointerUp (PointerEventData eventData)
	{
		if (!isActive)
			return;
		//mouseIsDown = true;
		Debug.Log (string.Format ("OnPointerClick ()"));
		Click ();
	}



	/// <summary>
	/// Removes the events.
	/// </summary>
	public void RemoveEvents ()
	{
		isActive = false;
		// make the collider size zero
		if (colliderRemoveFix)
			myBoxCollider.size = Vector3.zero;
	}

	/// <summary>
	/// On Switch Click
	/// </summary>
	void Click ()
	{
		//Play  Click Sound
		AudioManager.PlaySfx ("Click");

				 
		if (onClickComplete) {
			// remove all the events
			RemoveEvents ();

			//deactivate Hover State
			if (glowIsVisible)
				myGlow.SetActive (false);

			/// Show on Complete Objective Text
			if (FeedBackText != "")
				TopBarController.ShowObjective (FeedBackText, isCorrect, HRUtilities.ft);

			/// Send The Apropriate message
			if (isObjective)
				SendObjectiveCompleted ();
			else {
				SendTaskCompleted ();
				RemoveEvents ();
								
			}
						
		}
	}

	private BoxCollider _myBoxCollider;

	private BoxCollider myBoxCollider {
		get {
			if (_myBoxCollider == null)
				_myBoxCollider = GetComponent<BoxCollider> ();
			return _myBoxCollider;
		}   
		set {
			_myBoxCollider = value;
		}
	}



}
