﻿using UnityEngine;
using AudioSystem;

using System.Text;
using System.IO;

using System.Collections;
using System.Collections.Generic;

[System.Serializable]
/// <summary>
/// Text handler is Singleton Class tha handles all the text content of the game.
/// Tha class read's a specialy format text file that contains each scene text format.
/// each text has a specific code that where when its requested the game will server the appropreate text in right language
/// </summary>
public class TextHandler : MonoBehaviour
{
	/// <summary>
	/// Text file
	/// </summary>
	public TextAsset file;
	StreamReader sr;
	/// <summary>
	/// The raw text data
	/// </summary>
	string[] savedData;
	int lineCount = 0;

	/// <summary>
	/// All the 
	/// </summary>
	public List<TextClass> textList = new List<TextClass> ();

	public bool isDataLoaded = false;


	public void Awake ()
	{
		_instance = this;
	}

	void Start ()
	{
		isDataLoaded = Load (file);
	}

	/// <summary>
	/// Load the specified textfile.
	/// </summary>
	/// <param name="textfile">Textfile.</param>
	private bool Load (TextAsset textfile)
	{
		string line;
		//load the text file into a StringReader
		StringReader theReader = new StringReader (textfile.text); 
		using (theReader) {
			//for all the lines
			do {
				//we read the next line
				line = theReader.ReadLine ();
				// if not empty
				if (line != null) {
					// make a string array separate but tab space
					string[] entries = line.Split (new [] { '\t' });
					//if Array has contenet process the date
					if (entries.Length > 0 && entries [0] != "")
						proccessdata (entries, lineCount);
				}
				//inctrase the line count
				lineCount++;
			} while (line != null);
			// close the file strea,
			theReader.Close ();
			return true;
		}
	}



	/// <summary>
	/// Proccessdata the specified newData and line.
	/// </summary>
	/// <param name="newData">String Array </param>
	/// <param name="line">Current lint</param>
	private void proccessdata (string[] newData, int line)
	{
		//Ignore the first line
		if (line == 0)
			return;
		//Create a text entry
		TextClass newTextItem = new TextClass (newData [0]);
		for (int i = 1; i < newData.Length; i++) {
			//Add text per language
			newTextItem.AddText (newData [i].Replace ("<br>", "\n"));
		}
		//Add the text asset in the text list
		textList.Add (newTextItem);
	}


	/// <summary>
	/// Main method for requesting a text for the project
	/// </summary>
	/// <returns>The text.</returns>
	/// <param name="code">Idification Code</param>
	public string GetText (string code)
	{
		int langMode = 1;

				
		if (code == "")
			return "";

		code.Replace (" ", "");
		code.Replace ("/n", "");
		code.Replace ("/t", "");
		for (int i = 0; i < textList.Count; i++)
			if (textList [i].code == code)
				return textList [i].GetText (langMode);
						
		return code;
	}


	//Singleton definition.
	public static TextHandler _instance;

	public static TextHandler Instance {
		get {
			return _instance;
		}
	}

}