﻿using UnityEngine;
using UnityEngine.UI;
using AudioSystem;
using System.Collections;

/// <summary>
/// The Top bar is in the top of the screen and provides userfull information to the user in 3 ways.
/// General messages that provide the users with general information about
/// Objective messages that give information to player for Correct or Worng interactions
/// Hover messages provide information for in scene objects that the user is currently interacts
/// </summary>
public class TopBarController : MonoBehaviour
{

	public Color fontColor;
	public Color fontColor2;
	private Color fontColorAlpha;
	private Color fontColorAlpha2;

	/// <summary>
	/// The Display Position
	/// </summary>
	public Vector3 startPos;

	/// <summary>
	/// The Gameobject & text mesh tha display the text content
	/// </summary>
	private GameObject textMeshGO;
	private Text textMesh;
	private Text textMesh2;

	/// <summary>
	/// The thumb up/down Gameobject
	/// </summary>
	private GameObject ThumbGO;
	private SpriteRenderer ThumbSprite;

	/// <summary>
	/// The main & hover message currently in use
	/// </summary>
	private string mainMessage = "";
	//private string hoverMessage = "";

	/// <summary>
	/// Is in an objective view.
	/// </summary>
	private bool isObjectView = false;
	private bool isOnRotating = false;

	/// <summary>
	/// The Top Bar bottom Background.
	/// </summary>
	private Transform bottom;
	private SpriteRenderer bottomSprite;

	/// <summary>
	/// The default color.
	/// </summary>
	private Color defaultColor;
	/// <summary>
	/// The color of the hover.
	/// </summary>
	//private Color hoverColor;
	/// <summary>
	/// The color of the objective fail.
	/// </summary>
	//private Color objectiveFailColor;
	/// <summary>
	/// The color of the objective pass.
	/// </summary>
	//private Color objectivePassColor;

	/// <summary>
	/// A Singleton instance.
	/// </summary>
	private static TopBarController _instance;

	public static TopBarController Instance {
		get {
			if (_instance == null) {
				GameObject tbGO = GameObject.Find ("TopBar");
				if (tbGO)
					_instance = tbGO.GetComponent <TopBarController> ();
			}
			return _instance;
		}
	}


	void Awake ()
	{
		_instance = this;
		fontColorAlpha = fontColor;
		fontColorAlpha.a = 0;

		fontColorAlpha2 = fontColor;
		fontColorAlpha2.a = 0;

		//Define the colors
		//hoverColor = HRUtilities.Color01 (255, 234, 96);
		//objectiveFailColor = HRUtilities.Color01 (255, 121, 121);
		//objectivePassColor = HRUtilities.Color01 (121, 255, 127);

		//Capture Positions
		//endPos = myTransform.localPosition;
		//myTransform.localPosition = startPos;

		//Fix the Background
		//bottom = myTransform.FindChild ("bottom");
		//bottomSprite = bottom.GetComponent <SpriteRenderer> ();

		//Capture the default color
		//defaultColor = bottomSprite.color;
		// get the text mesh
		textMeshGO = myTransform.FindChild ("txtMesh").gameObject;

		textMesh = textMeshGO.GetComponent <Text> ();
		textMesh2 = myTransform.FindChild ("txtMesh_2").gameObject.GetComponent <Text> ();
		// get the thumb gameobject
		//ThumbGO = myTransform.FindChild ("Thumb").gameObject;
		//ThumbSprite = ThumbGO.GetComponent <SpriteRenderer> ();
		//ThumbGO.transform.localScale = Vector3.zero;
		//ThumbGO.SetActive (false);
	}

		
	#region  Main Message Section

	/// <summary>
	/// Shows a Main message in the bar
	/// </summary>
	/// <param name="newText">New text.</param>
	public void ShowBarMsg (string newText = "")
	{
		Debug.Log (string.Format ("TopBar ShowBarMsg : {0}", newText));
		// if the text was empty dont do anything
		if (newText != "") {
			// get converted message
			string uncodedMsg = TextHandler.Instance.GetText (newText);
			//store the text
			mainMessage = uncodedMsg;
			//set the text mesh
			textMesh2.text = textMesh.text = uncodedMsg;	
			// play a Sfx
			AudioManager.PlaySfx ("NewTask");

			//Set the bar to its default color in case it was displaying something else
			//bottomSprite.color = defaultColor;

			//Animated it
			//StartCoroutine (rotationalShake ());
			//StartCoroutine (myTransform.MoveTo (endPos, HRUtilities.at, EaseType.BackOut));
			StartCoroutine (FadeText (textMesh, 0, fontColorAlpha, fontColor));
			StartCoroutine (FadeText (textMesh2, 0, fontColorAlpha2, fontColor2));
		} 
	}

	IEnumerator FadeText (Text text, float duration, Color from, Color to)
	{

		float elapsed = 0;
		float start = 0;
		float range = 1;
		float time = 0;


		while (elapsed < duration) {
			elapsed = Mathf.MoveTowards (elapsed, duration, Time.deltaTime);
			time = start + range * (elapsed / duration);
			//Debug.Log (string.Format ("time : {0}", time));
			text.color = Color.Lerp (from, to, time);
			yield return 0;
		}

		text.color = to;
	}



	/// <summary>
	/// Reverts the store message.
	/// </summary>
	public void RevertBarMsg ()
	{
		mainMessage = "";
	}

	/// <summary>
	/// Hides the bar message delay.
	/// </summary>
	/// <returns>The bar message delay.</returns>
	/// <param name="time">Desired Time.</param>
	public IEnumerator HideBarMsgDelay (float time)
	{
		yield return new WaitForSeconds (time);
		HideBarMsg ();
	}

	/// <summary>
	/// Hides the bar message.
	/// </summary>
	public void HideBarMsg ()
	{
		//mainMessage = "";
		//textMesh.text = "";	

		StartCoroutine (FadeText (textMesh, 0, fontColor, fontColorAlpha));
		StartCoroutine (FadeText (textMesh2, 0, fontColor2, fontColorAlpha2));

		//StartCoroutine (myTransform.MoveTo (startPos, HRUtilities.at, EaseType.BackOut));
	}

	/// <summary>
	/// Shake Animation
	/// </summary>
	IEnumerator rotationalShake ()
	{
		//set isOnRotating true to block any other animations
		isOnRotating = true;
		for (int i = 0; i < 3; i++) {
			yield return StartCoroutine (myTransform.RotateTo (Quaternion.Euler (new Vector3 (0, 0, 5)), HRUtilities.at / 8, EaseType.QuadOut));
			yield return StartCoroutine (myTransform.RotateTo (Quaternion.Euler (new Vector3 (0, 0, -5)), HRUtilities.at / 4, EaseType.QuadInOut));
			yield return StartCoroutine (myTransform.RotateTo (Quaternion.Euler (new Vector3 (0, 0, 0)), HRUtilities.at / 8, EaseType.QuadIn));
		}
		isOnRotating = false;
	}

	/// <summary>
	/// Show the bar Static function
	/// </summary>
	/// <param name="newText">New text.</param>
	public static void ShowBar (string newText = "")
	{

		if (Instance != null)
			Instance.ShowBarMsg (newText);
	}

	/// <summary>
	/// Hide the bar Static function.
	/// </summary>
	/// <param name="time">Time.</param>
	public static void HideBar (float time = 0)
	{
		if (Instance != null) {
			if (time == 0)
				Instance.HideBarMsg ();
			else
				Instance.StartCoroutine (Instance.HideBarMsgDelay (time));
		}
	}

	/// <summary>
	/// Revert the bar Static Function
	/// </summary>
	public static void RevertBar ()
	{
		if (Instance != null)
			Instance.RevertBarMsg ();

	}

	#endregion



	#region  Hover Message Section

	/// <summary>
	/// Shows the A hover message on the topbar.
	/// </summary>
	/// <param name="newText">New text.</param>
	/// <param name="revertTime">if Revert time if zero wait for specific hide command</param>
	public void ShowHoverMsg (string newText = "", float revertTime = 0)
	{
		// If currently we are showing and objective message dont do anything
		if (isObjectView)
			return;
		//Store the message info
		//hoverMessage = newText;
		textMesh2.text = textMesh.text = newText;	
		//Change the color
		//bottomSprite.color = hoverColor;
		//Animate it
		StartCoroutine (myTransform.ScaleTo (Vector3.one * 1.01f, HRUtilities.at / 3, EaseType.BackIn));
		//revert to previous state
		if (revertTime != 0)
			StartCoroutine (RevertHoverMsg (revertTime));
	}

		
	/// <summary>
	/// Hides the hover message.
	/// </summary>
	public void HideHoverMsg ()
	{
		// If currently we are showing and objective message dont do anything
		if (isObjectView)
			return;
		//Animate the bar
		StartCoroutine (myTransform.ScaleTo (Vector3.one, HRUtilities.at / 3, EaseType.BackOut));
		//Change the background color
		//bottomSprite.color = defaultColor;
		//Set the text
		textMesh2.text = textMesh.text = mainMessage;	
	}

	/// <summary>
	/// Reverts the hover message.
	/// </summary>
	/// <param name="revertTime">Revert time.</param>
	IEnumerator RevertHoverMsg (float revertTime)
	{
		if (!isObjectView) {
			yield return StartCoroutine (Auto.Wait (revertTime));
			HideHoverMsg ();
		}
	}


	/// <summary>
	/// Static function that Shows the hover.
	/// </summary>
	/// <param name="newText">New text.</param>
	/// <param name="revertTime">Revert time.</param>
	public static void ShowHover (string newText = "", float revertTime = 0)
	{
		if (Instance != null && newText != "")
			Instance.ShowHoverMsg (newText, revertTime);
	}

	/// <summary>
	/// Static function that Hides the hover.
	/// </summary>
	public static void HideHover ()
	{
		if (Instance != null)
			Instance.HideHoverMsg ();
		
	}

	#endregion




	/* ------------------   Objective Message Section   ------------------*/

	#region  Objective Message Section

	/// <summary>
	/// Shows an objective Message.
	/// </summary>
	/// <param name="newText">New text.</param>
	/// <param name="isGood">If set to <c>true</c> is good.</param>
	/// <param name="revertTime">Revert time.</param>
	public void ShowObjectiveStatus (string newText = "", bool isGood = false, float revertTime = 0)
	{
		// set the text
		textMesh2.text = textMesh.text = newText;	

		//enable the thumb GO
		//ThumbGO.SetActive (true);
		//Set the status in objective view
		isObjectView = true;
		//Stop any other animation tha may be happening
		StopAllCoroutines ();


		if (isGood) {
			//Play success sfx
			AudioManager.PlaySfx ("Win");
			//Set the bg Color
			//bottomSprite.color = objectivePassColor;
			//Animate
			StartCoroutine (ScaleJump ());
		} else {
			AudioManager.PlaySfx ("Fail");
			//bottomSprite.color = objectiveFailColor;
			StartCoroutine (ShakeHover ());
		}
		//Animate Thumb status 
		StartCoroutine (AnimateStatus (revertTime));
	}

	/// <summary>
	/// Animates the Objectiva icon.
	/// </summary>
	/// <param name="revertTIme">Revert TIME.</param>
	IEnumerator AnimateStatus (float revertTIme)
	{
		//if we are alredy rotation set the bar str8
		if (isOnRotating) {
			yield return StartCoroutine (myTransform.RotateTo (Quaternion.Euler (new Vector3 (0, 0, 0)), HRUtilities.at / 4, EaseType.QuadOut));
			isOnRotating = false;
		}

		//Animate the thumb icon
		//yield return StartCoroutine (ThumbGO.transform.ScaleTo (Vector3.one, HRUtilities.at, EaseType.BackOut));
		//yield return StartCoroutine (Auto.Wait (revertTIme));
		//yield return StartCoroutine (ThumbGO.transform.ScaleTo (Vector3.zero, HRUtilities.at, EaseType.BackIn));

		//revert the bar
		isObjectView = false;
		//bottomSprite.color = defaultColor;
		textMesh2.text = textMesh.text = mainMessage;	

	}

	/// <summary>
	/// Static function that Shows the objective.
	/// </summary>
	/// <param name="newText">New text.</param>
	/// <param name="isGood">If set to <c>true</c> is good.</param>
	/// <param name="revertTime">Revert time.</param>
	public static void ShowObjective (string newText = "", bool isGood = false, float revertTime = 0)
	{
		if (Instance != null)
			Instance.ShowObjectiveStatus (newText, isGood, revertTime);
	}

	#endregion

	/// <summary>
	/// Simple shake animation
	/// </summary>
	IEnumerator ShakeHover ()
	{
		Vector3 startPos = myTransform.position;
		yield return StartCoroutine (myTransform.Shake (0.35f, HRUtilities.at / 2));
		myTransform.position = startPos;
	}

	/// <summary>
	/// Simple Scale animation
	/// </summary>
	/// <returns>The jump.</returns>
	IEnumerator ScaleJump ()
	{
		yield return StartCoroutine (myTransform.ScaleTo (Vector3.one * 1.1f, HRUtilities.at / 3, EaseType.BackIn));
		yield return StartCoroutine (myTransform.ScaleTo (Vector3.one, HRUtilities.at / 3, EaseType.BackOut));
	}

	private Transform _myTransform;

	private Transform myTransform {
		get {
			if (_myTransform == null)
				_myTransform = GetComponent<Transform> ();
			return _myTransform;
		}
		
		set {
			_myTransform = value;
		}
	}

	
}
