﻿using UnityEngine;
using AudioSystem;
using System.Collections;

/// <summary>
/// Simple utilitie class that upon activation will cause set the gameobject to a looping animation state
/// </summary>
public class SimpleAnimation : TaskHandler
{
	[Header ("Animation settings")]
	[Range (-5, 5)]
	public float
		AnimationAmount = 1.2f;

	[Range (0, 10)]
	public float 
		timeMultiplier = 1;
		
	public SimpleAnimationType animation;
		
	private Transform myTransform;
	private Vector3 startPos;
	private Vector3 startScale;

	private SpriteRenderer mySprite;
		
	private string currCoRoutine;

	[Header ("Events Settings")]
	public bool startOnEnable = false;
	public bool startOnAwake = false;
	public bool startOnObjective = false;

		

	public void Awake ()
	{
		myTransform = transform;
		startPos = myTransform.localPosition;
		startScale = myTransform.localScale;
	}

	public void Start ()
	{
		if (startOnAwake)
			EnableTask ();
	}

	public void OnEnable ()
	{
		if (startOnEnable)
			EnableTask ();
	}

	public void EnableObjective ()
	{
		if (startOnObjective)
			EnableTask ();
	}




	public override void EnableTask ()
	{
		StartCoroutine (StartWithRandomDelay ());
	}


	IEnumerator StartWithRandomDelay ()
	{

		float waitTime = Random.Range (0f, 2f);

		yield return new WaitForSeconds (waitTime);

		switch (animation) {
		case SimpleAnimationType.SlideUpDown:
			currCoRoutine = "SlideAnimation";
			break;
		case SimpleAnimationType.SlideLeftRight:
			currCoRoutine = "SlideAnimation2";
			break;
		case SimpleAnimationType.ZoomInOut:
			currCoRoutine = "ZoomAnimation";
			break;
		case SimpleAnimationType.FadeInOutAlpha:
			mySprite = GetComponentInChildren <SpriteRenderer> ();
			currCoRoutine = "AlphaAnimation";
			break;
		}
		
		StartCoroutine (currCoRoutine);
	}


	public void OnDisable ()
	{
		StopCoroutine (currCoRoutine);
	}

	public void TaskCompleted ()
	{
		StopCoroutine (currCoRoutine);
	}


	IEnumerator SlideAnimation ()
	{
		while (true) {
			yield return StartCoroutine (myTransform.MoveTo (new Vector3 (startPos.x, startPos.y + AnimationAmount, 0), HRUtilities.at * timeMultiplier, EaseType.QuadInOut));
			yield return StartCoroutine (myTransform.MoveTo (new Vector3 (startPos.x, startPos.y, 0), HRUtilities.at * timeMultiplier, EaseType.QuadInOut));
		}
	}


	IEnumerator SlideAnimation2 ()
	{
		while (true) {
			yield return StartCoroutine (myTransform.MoveTo (new Vector3 (startPos.x + AnimationAmount, startPos.y, 0), HRUtilities.at * timeMultiplier, EaseType.QuadInOut));
			yield return StartCoroutine (myTransform.MoveTo (new Vector3 (startPos.x, startPos.y, 0), HRUtilities.at * timeMultiplier, EaseType.QuadInOut));
			//yield return StartCoroutine (myTransform.MoveTo (new Vector3 (startPos.x, startPos.y - AnimationAmount, 0), HRUtilities.at, EaseType.QuadIn));
		}
	}

	IEnumerator ZoomAnimation ()
	{
		while (true) {
			yield return StartCoroutine (myTransform.ScaleTo (startScale * AnimationAmount, HRUtilities.at * timeMultiplier));
			yield return StartCoroutine (myTransform.ScaleTo (startScale, HRUtilities.at * timeMultiplier));
			//yield return StartCoroutine (myTransform.MoveTo (new Vector3 (startPos.x, startPos.y - AnimationAmount, 0), HRUtilities.at, EaseType.QuadIn));
		}
	}


	IEnumerator AlphaAnimation ()
	{

		while (true) {
			yield return StartCoroutine (mySprite.FadeAlphaTime (0, HRUtilities.at, EaseType.Linear));
			yield return StartCoroutine (mySprite.FadeAlphaTime (1, HRUtilities.at, EaseType.Linear));
		}
	}



}




public enum SimpleAnimationType
{
	ZoomInOut,
	SlideUpDown,
	FadeInOutAlpha,
	SlideLeftRight}
;
