﻿using UnityEngine;
using AudioSystem;
using System.Collections;

/// <summary>
/// Simple Utilitie Class that can be added to any Game object and when activate it will cause it to shake.
/// </summary>
public class SimpleShake : TaskHandler
{
		[Header ("Event Settings")]
		public bool startOnEnable = false;
		public bool startOnAwake = false;
		public bool startOnObjective = false;
		[Header ("Main Settings")]
		public bool isSingleShake = false;
		public string OnEnableSound = "";
		public float ShakeAmount = 0.2f;

		private bool isEnabled = false;

		public void Start ()
		{
				if (startOnAwake)
						EnableTask ();
		}

		public void OnEnable ()
		{
				if (startOnEnable)
						EnableTask ();
		}

		public void EnableObjective ()
		{
				if (startOnObjective)
						EnableTask ();
		}

		public override void EnableTask ()
		{				
				isEnabled = true;
				
				StartCoroutine (ShakeAnimate ());
		}


		IEnumerator ShakeAnimate ()
		{
				while (isEnabled) {
						if (OnEnableSound != "")
								AudioManager.PlaySfx (OnEnableSound);
						yield return StartCoroutine (myTransform.Shake (ShakeAmount, HRUtilities.at * 2));
						yield return StartCoroutine (Auto.Wait (HRUtilities.at * 4));
						if (isSingleShake)
								isEnabled = false;
				}
		}

		
		public void ObjectiveCompleted ()
		{
				isEnabled = false;
		}

		public void OnDisable ()
		{
				ObjectiveCompleted ();
		}


		private Transform _myTransform;

		private Transform myTransform {
				get {
						if (_myTransform == null)
								_myTransform = GetComponent<Transform> ();
						return _myTransform;
				}
		
				set {
						_myTransform = value;
				}
		
		}


}
