﻿using UnityEngine;
using AudioSystem;
using System.Collections;

/// <summary>
/// Transition handler is a module that attaches to the Room Controller and handles transions in our game.
/// it can zoom, slide, fade in and out a selected object.
/// </summary>
public class TransitionHandler : MonoBehaviour
{
	public void AnimateObject (GameObject goItem, AnimationType animType)
	{

		//Debug.Log (string.Format ("AnimateObject () - item: {0} , type:{1}", goItem.name, animType.ToString ()));

		goItem.SetActive (true);

		switch (animType) {
		case AnimationType.ZoomIn:
			goItem.transform.localScale = Vector3.zero;
			StartCoroutine (ScaleIn (goItem.transform));
			break;

		case AnimationType.ZoomOut:
			StartCoroutine (ScaleOut (goItem.transform));
			break;
			
		case AnimationType.SlideInUp:
			goItem.transform.localPosition = goItem.transform.localPosition + new Vector3 (0, 19, 0);
			StartCoroutine (SlideInUp (goItem.transform));
			break;

		case AnimationType.SlideOutUp:
			StartCoroutine (SlideOutUp (goItem.transform));
			break;
				
		case AnimationType.FadeInAlpha:
			StartCoroutine (FadeInAlpha (goItem.transform));
			break;

		case AnimationType.FadeOutAlpha:
			StartCoroutine (FadeOutAlpha (goItem.transform));
			break;
		}
	}


	IEnumerator ScaleIn (Transform transItem)
	{
		yield return StartCoroutine (transItem.ScaleTo (Vector3.one, HRUtilities.at * 2, EaseType.BackOut));
		AnimationComplete ();
	}

	IEnumerator ScaleOut (Transform transItem)
	{
		Debug.Log (string.Format ("ScaleOut () - Start ({0})", Time.time));
		yield return StartCoroutine (transItem.ScaleTo (Vector3.zero, HRUtilities.at * 2, EaseType.BackIn));
		AnimationComplete ();
		Debug.Log (string.Format ("ScaleOut () - End ({0})", Time.time));
	}

	IEnumerator SlideInUp (Transform transItem)
	{
		//yield return StartCoroutine (transItem.MoveToGlobal (transItem.position - new Vector3 (0, 19, 0), HRUtilities.at * 2, EaseType.BackOut));
		yield return StartCoroutine (transItem.MoveTo (transItem.localPosition - new Vector3 (0, 19, 0), HRUtilities.at * 2, EaseType.BackOut));
		AnimationComplete ();
	}

	IEnumerator SlideOutUp (Transform transItem)
	{
		yield return StartCoroutine (transItem.MoveTo (transItem.localPosition + new Vector3 (0, 19, 0), HRUtilities.at * 2, EaseType.BackIn));
		AnimationComplete ();
	}


	IEnumerator FadeInAlpha (Transform transItem)
	{

		SpriteRenderer[] spriteList = transItem.GetComponentsInChildren <SpriteRenderer> ();

		for (int i = 0; i < spriteList.Length; i++) {
			spriteList [i].color = HRUtilities.fA;
		}
			
		for (int i = 0; i < spriteList.Length; i++) {
			if (i < spriteList.Length - 1)
				StartCoroutine (spriteList [i].FadeAlphaTime (1, HRUtilities.at, EaseType.Linear));
			else
				yield return StartCoroutine (spriteList [i].FadeAlphaTime (1, HRUtilities.at, EaseType.Linear));
		}
				

		AnimationComplete ();
	}

	IEnumerator FadeOutAlpha (Transform transItem)
	{
		SpriteRenderer[] spriteList = transItem.GetComponentsInChildren <SpriteRenderer> ();
		for (int i = 0; i < spriteList.Length; i++) {
			if (i < spriteList.Length - 1)
				StartCoroutine (spriteList [i].FadeAlphaTime (0, HRUtilities.at, EaseType.Linear));
			else
				yield return StartCoroutine (spriteList [i].FadeAlphaTime (0, HRUtilities.at, EaseType.Linear));
		}
		AnimationComplete ();
	}



	void AnimationComplete ()
	{
		SendMessageUpwards (HRUtilities.TaskCompleted, gameObject, SendMessageOptions.DontRequireReceiver);
	}



}


public class AnimatedObj
{
	public GameObject animatedGO;
	public Vector3 startPos;
	public Vector3 startScale;
	public Quaternion startRotation;

}



public enum AnimationType
{
	ZoomIn,
	ZoomOut,
	SlideInUp,
	SlideOutUp,
	FadeInAlpha,
	FadeOutAlpha}
;