﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class AtmospherePositionControl : TaskHandler
{


	public float time;

	[Header ("Sfx Settings")]
	private float startStartSfXVol;
	private float stratEndSfXVol;
	public AudioSource startSfX;
	public AudioSource endSfX;


	[Header ("Debug Settings")]

	public bool debugMode = true;

	[Header ("Generic Settings")]
	public float minTime;
	public float maxTime;
	[Range (0, 1)]
	public float percResult;

	public List<EviormentalStyle> enviormentTimeline;

	public ParticleSystem[] particlesSystems;

	public ParticleSystem wind;
	public Gradient windColoring;

	[Header ("Sea Timeline")]
	public OceanClass startSea;
	public OceanClass endSea;


	public float climeForceStart = 0f;
	public float climeForceEnd = 0.09f;


	[Header ("Scene Clouds")]
	public ParticleSystem clouds;
	public float startEmissionRate;
	public float endEmissionRate;

	public ParticleSystem.MinMaxCurve emissionRate;
	public ParticleSystem.EmissionModule emission;


	public static AtmospherePositionControl instance;

	public void Awake ()
	{
		instance = this;
	}

	public void Start ()
	{
		startStartSfXVol = startSfX.volume;
		stratEndSfXVol = endSfX.volume;
		StartCoroutine (UpdateAtmosphere ());
	}


	public override void EnableTask ()
	{
		StartCoroutine (UpdateAtmosphere ());
		SendTaskCompleted ();

	}

	private float gameStarted;

	public void StartGame ()
	{
		if (debugMode) {
			debugMode = false;
			gameStarted = Time.time;
		}
	}


	IEnumerator UpdateAtmosphere ()
	{
		while (true) {

			if (!debugMode)
				percResult = time = Mathf.InverseLerp (minTime, maxTime, Time.time - gameStarted);
			else
				time = percResult;


			OceanHandler.instance.SetSea (startSea, endSea, time);
		
			SetAtmosphereTimeLine (time);

			//BoatHandler (time);

			//PoseidonHandler (time);
			SoundHandling (time);
			ParticleHandler (time);


			yield return new WaitForFixedUpdate ();
		}
	}

	public void SoundHandling (float time)
	{

		startSfX.volume = (1 - time) * startStartSfXVol;
		endSfX.volume = time * stratEndSfXVol;
	}


	public void ParticleHandler (float time)
	{
		wind.startColor = windColoring.Evaluate (time);

		for (int i = 0; i < particlesSystems.Length; i++)
			particlesSystems [i].startSize = time;
		

		float emissionRate = Mathf.Lerp (startEmissionRate, endEmissionRate, time);
		var emission = clouds.emission;
		var rate = emission.rate;
		rate.constant = emissionRate;
		emission.rate = rate;
	}

	 


	public void SetAtmosphereTimeLine (float time)
	{
		int perc = Mathf.RoundToInt (time * 1000);

		int stepWeight = 1000 / (enviormentTimeline.Count - 1);

		int currentStep;
		if (perc == 0)
			currentStep = 0;
		else
			currentStep = perc / stepWeight;

		int fromID = currentStep;
		int toID = currentStep + 1;

		float innerPerc = -(((float)currentStep * (float)stepWeight) - (float)perc) / (float)stepWeight;


		if (toID == enviormentTimeline.Count)
			return;

		EnviormentHandler.instance.SetAtmosphere (enviormentTimeline [fromID], enviormentTimeline [toID], innerPerc);
	}



	private Transform _myTransform;

	private Transform myTransform {
		get {
			if (_myTransform == null)
				_myTransform = GetComponent<Transform> ();
			return _myTransform;
		}

		set {
			_myTransform = value;
		}

	}

}
