﻿using UnityEngine;
using System.Collections;
using Rewired;
using AudioSystem;

public class PlayerControl : MonoBehaviour
{

	[Header ("Game Settings")]
	//public float score;

	public bool isActive;
	[Header ("Control Settings")]

	public int playerId = 0;
	private Player player;


	[HideInInspector]
	public bool facingRight = true;
	// For determining which way the player is currently facing.
	[HideInInspector]
	public bool jump = false;
	// Condition for whether the player should jump.


	public float moveForce = 365f;
	// Amount of force added to move the player left and right.
	public float maxSpeed = 5f;
	// The fastest the player can travel in the x axis.
	public AudioClip[] jumpClips;
	// Array of clips for when the player jumps.
	public float jumpForce = 1000f;
	// Amount of force added when the player jumps.


	private Transform groundCheck;
	// A position marking where to check if the player is grounded.
	public bool grounded = false;
	// Whether or not the player is grounded.
	private Animator anim;
	// Reference to the player's animator component.


	void Awake ()
	{
		// Setting up references.
		groundCheck = transform.Find ("groundCheck");
		anim = GetComponent<Animator> ();
	}

	void Start ()
	{
		player = ReInput.players.GetPlayer (playerId);
	}


	public void AddScore (float amount)
	{
		score += amount;
		int rndID = Random.Range (0, 6);
		AudioManager.PlaySfx ("PickUp" + rndID);

	}

	void Update ()
	{


		if (!isActive || GameController.instance.boatFlipped) {
			return;
		}
		// The player is grounded if a linecast to the groundcheck position hits anything on the ground layer.


		if (!grounded) {
			
			grounded = Physics2D.Linecast (transform.position, groundCheck.position, 1 << LayerMask.NameToLayer ("Ground"));  
			if (grounded)
				anim.SetBool ("Jump", false);
			
		}
		// If the jump button is pressed and the player is grounded then the player should jump.

		if (player.GetButtonDown ("Jump") && grounded) {
			jump = true;
			anim.SetBool ("Jump", true);
		}
	}

	public float hitOffeset;

	void FixedUpdate ()
	{

		if (GameController.instance.boatFlipped) {
			return;
		}
		

		if (!isActive) {

			if (player.GetButtonDown ("Jump")) {
				RespawnPlayer ();
				isActive = true;
				AudioManager.PlaySfx ("COLISION " + playerId);
				AtmospherePositionControl.instance.StartGame ();
			}


			return;
		}
		// Cache the horizontal input.
		//float h = Input.GetAxis ("Horizontal");
		float h = player.GetAxis ("MoveX");

		// The Speed animator parameter is set to the absolute value of the horizontal input.
		anim.SetFloat ("Speed", Mathf.Abs (h) * 2);
		//anim.SetFloat ("Speed", Mathf.Abs (myRigidbody2D.velocity.x) * 2);
		// If the player is changing direction (h has a different sign to velocity.x) or hasn't reached maxSpeed yet...
		if (h * myRigidbody2D.velocity.x < maxSpeed)
			// ... add a force to the player.
			myRigidbody2D.AddForce (Vector2.right * h * moveForce);

		// If the player's horizontal velocity is greater than the maxSpeed...
		if (Mathf.Abs (myRigidbody2D.velocity.x) > maxSpeed)
			// ... set the player's velocity to the maxSpeed in the x axis.
			myRigidbody2D.velocity = new Vector2 (Mathf.Sign (myRigidbody2D.velocity.x) * maxSpeed, myRigidbody2D.velocity.y);





		// If the input is moving the player right and the player is facing left...
		if (h > 0 && !facingRight)
			// ... flip the player.
			Flip ();
		// Otherwise if the input is moving the player left and the player is facing right...
		else if (h < 0 && facingRight)
			// ... flip the player.
			Flip ();

		// If the player should jump...
		if (jump) {
			grounded = false;
			// Set the Jump animator trigger parameter.
			//anim.SetTrigger ("Jump");
			//anim.SetBool ("Jump", true);
			// Play a random jump audio clip.
			int i = Random.Range (0, jumpClips.Length);
			AudioSource.PlayClipAtPoint (jumpClips [i], transform.position);

			// Add a vertical force to the player.
			myRigidbody2D.AddForce (new Vector2 (0f, jumpForce));

			// Make sure the player can't jump again until the jump conditions from Update are satisfied.
			jump = false;
		}
			

		/*
		if (grounded) {
			Vector2 vel = myRigidbody2D.velocity;
			vel.y = 0;
			RaycastHit2D hit = Physics2D.Raycast (myRigidbody2D.position + vel, Vector2.down, 10, 1 << LayerMask.NameToLayer ("Ground"));
			Debug.Log (string.Format ("hit.transform : {0}", hit.transform));
			//Debug.Log (string.Format ("hit.point : {0}", hit.point));
			Vector2 pos = myRigidbody2D.position;
			pos.y = hit.point.y - hitOffeset;
			Debug.Log (string.Format ("pos : {0}", pos));
			myRigidbody2D.MovePosition (pos);// = pos;
		}
		*/

		if (myRigidbody2D.position.y < ReSpawnHandler.instance.deathDepth) {
			RespawnPlayer ();
			int randomID = Random.Range (0, 2);
			AudioManager.PlaySfx ("Death" + randomID);
		}

	}



	public void RespawnPlayer ()
	{

		score -= GameController.instance.deathCost;

		Vector3 newPosition = ReSpawnHandler.instance.RequestNewSpawnPoint ();
		myTransform.position = newPosition;
		myRigidbody2D.velocity = Vector2.zero;
		anim.SetFloat ("Speed", 0);

		grounded = false;
		jump = false;
		anim.SetBool ("Jump", false);
	}

	
	void Flip ()
	{
		// Switch the way the player is labelled as facing.
		facingRight = !facingRight;

		// Multiply the player's x local scale by -1.
		Vector3 theScale = transform.localScale;
		theScale.x *= -1;
		transform.localScale = theScale;
	}

	private Rigidbody2D _myRigidbody2D;

	public Rigidbody2D myRigidbody2D {
		get {
			if (_myRigidbody2D == null)
				_myRigidbody2D = GetComponent<Rigidbody2D> ();
			return _myRigidbody2D;
		}   
		set {
			_myRigidbody2D = value;
		}
	}

	private Transform _myTransform;

	private Transform myTransform {
		get {
			if (_myTransform == null)
				_myTransform = GetComponent<Transform> ();
			return _myTransform;
		}

		set {
			_myTransform = value;
		}

	}



	private float _score;

	public float score {
		get {
			return _score;
		}   
		set {
			_score = Mathf.Clamp (value, 0, Mathf.Infinity);
		}
	}

}
